@@ -18,11 +18,11 @@
 from __future__ import absolute_import
 
 import errno
-import gobject
 import locale
 import logging
 import os
 import shutil
+import sqlite3
 import subprocess
 import sys
 import tempfile
@@ -44,6 +44,11 @@ from solfege import utils
 
 import solfege
 
+try:
+    from pyalsa import alsaseq
+except ImportError:
+    alsaseq = None
+
 solfege_copyright = u"Copyright © 1999-2008 Tom Cato Amundsen <tca@gnu.org>, and others."
 
 warranty = """
@@ -72,7 +77,7 @@ def check_rcfile():
         return
     if cfg.get_int("app/rcfileversion") <= 1:
         if not "example-files" in cfg.get_string('config/lessoncollections'):
-            cfg.set_string('config/lessoncollections', 
+            cfg.set_string('config/lessoncollections',
                 "%s example-files" % cfg.get_string('config/lessoncollections'))
     if cfg.get_int("app/rcfileversion") <= 5:
         # This is more complicated that necessary to fix an old
@@ -139,7 +144,7 @@ def check_rcfile():
     if cfg.get_int("app/rcfileversion") < 17:
         v = cfg.get_string("app/frontpage").split("/")
         if v[0] == u"exercises" and v[1] != u"standard":
-            cfg.set_string("app/frontpage", 
+            cfg.set_string("app/frontpage",
                            u"/".join([v[0], u"standard"] + v[1:]))
     if cfg.get_int("app/rcfileversion") < 18:
         cfg.del_key("gui/web_browser_as_help_browser")
@@ -191,6 +196,29 @@ class SolfegeApp(cfg.ConfigUtils):
         if self.m_options.no_sound \
            or cfg.get_string("sound/type") == "fake-synth":
             soundcard.initialise_using_fake_synth(self.m_options.verbose_sound_init)
+        elif cfg.get_string("sound/type") == "alsa-sequencer":
+            if alsaseq:
+                try:
+                    clientid, portid = self.get_list("sound/alsa-client-port")
+                except ValueError:
+                    clientid, portid = (None, None)
+                try:
+                    soundcard.initialise_alsa_sequencer((clientid, portid),
+                            self.m_options.verbose_sound_init)
+                except alsaseq.SequencerError, e:
+                    logging.debug("initialise_alsa_sequencer failed. Using fake synth.")
+                    self.display_sound_init_error_message(e)
+                    soundcard.initialise_using_fake_synth(True)
+                    return
+            else:
+                if solfege.splash_win:
+                    solfege.splash_win.hide()
+                gu.dialog_ok(_("The pyalsa python module is missing"),
+                    solfege.win,
+                    _("Solfege was configured to the Python modules from www.alsa-project.org, but the modules was not found. You must reconfigure sound in the preferences window (Ctrl-F12) or restart Solfege in a way that it finds the module."))
+                soundcard.initialise_using_fake_synth(True)
+                if solfege.splash_win:
+                    solfege.splash_win.show()
         elif cfg.get_string("sound/type") == "winsynth":
             try:
                 soundcard.initialise_winsynth(cfg.get_int("sound/synth_number"),
@@ -213,7 +241,6 @@ class SolfegeApp(cfg.ConfigUtils):
                 cfg.set_int("sound/synth_number", soundcard.synth.m_devnum)
         elif cfg.get_string("sound/type") == "external-midiplayer":
             soundcard.initialise_external_midiplayer(
-                    cfg.get_string("sound/midi_player"),
                     verbose_init=self.m_options.verbose_sound_init)
             soundcard.synth.error_report_cb = solfege.win.display_error_message
         elif cfg.get_string("sound/type") == '':
@@ -251,13 +278,13 @@ class SolfegeApp(cfg.ConfigUtils):
             """%s""" % str(e).decode(locale.getpreferredencoding(), 'replace'))
         elif isinstance(e, ImportError):
             solfege.win.display_error_message2(str(e), _("You should configure sound from the preferences window, and try to use an external midi player. Or try to recompile the program and check for error messages to see why the module is not built."))
-        elif e.errno == errno.EACCES:
+        elif getattr(e, 'errno', None) == errno.EACCES:
             solfege.win.display_error_message(
                 "The sound init failed: %s\n"
                 "The errno EACCES indicates that you don't have write "
                 "permission to the device."
                 % str(e).decode(locale.getpreferredencoding(), 'replace'))
-        elif e.errno == errno.EBUSY:
+        elif getattr(e, 'errno', None) == errno.EBUSY:
             solfege.win.display_error_message(
                 "The sound init failed: %s\n"
                 "It seems like some other program is using the device. You "
@@ -373,8 +400,14 @@ class SolfegeApp(cfg.ConfigUtils):
             # exceptions.
             if g.m_t.m_P and getattr(g.m_t.m_P, 'header', None):
                 g.on_end_practise()
-        solfege.db.conn.commit()
-        solfege.db.conn.close()
+        try:
+            solfege.db.conn.commit()
+        except sqlite3.ProgrammingError, e:
+            gu.display_exception_message(e)
+        try:
+            solfege.db.conn.close()
+        except sqlite3.ProgrammingError, e:
+            gu.display_exception_message(e)
         if soundcard.synth:
             soundcard.synth.close()
         shutil.rmtree(lessonfile.MusicBaseClass.temp_dir, True)
@@ -453,7 +486,7 @@ class SolfegeApp(cfg.ConfigUtils):
                                     (self.get_int('config/default_bpm'), 4))
                     soundcard.end_export()
                 else:
-                    logging.warning("export_training_set:ignoring exercise with module='%s'" % module)
+                    logging.warning("export_training_set:ignoring exercise with module='%s'", module)
 #####
                 def do_convert(from_format, to_format):
                     """
