@@ -23,7 +23,26 @@ import pango
 from solfege.mpd import const
 from solfege.mpd import elems
 from solfege.mpd import mpdutils
-from solfege.mpd import MusicalPitch, Rat
+from solfege.mpd.musicalpitch import MusicalPitch
+
+ACCIDENTAL__2 = -2
+ACCIDENTAL__1 = -1
+ACCIDENTAL_0 = 0
+ACCIDENTAL_1 = 1
+ACCIDENTAL_2 = 2
+
+NOTEHEAD_WHOLE = 0
+NOTEHEAD_HALF = 1
+NOTEHEAD_QUARTER = 2
+
+FLAG_8UP = 'u3'
+FLAG_8DOWN = 'd3'
+FLAG_16UP = 'u4'
+FLAG_16DOWN = 'd4'
+FLAG_32UP = 'u5'
+FLAG_32DOWN = 'd5'
+FLAG_64UP = 'u6'
+FLAG_64DOWN = 'd6'
 
 fetadir = "feta"
 
@@ -45,11 +64,11 @@ class dim20:
 
 dimentions = {20: dim20}
 
-accidental_y_offset = {const.ACCIDENTAL__2: -7,
-                       const.ACCIDENTAL__1: -7,
-                       const.ACCIDENTAL_0: -5,
-                       const.ACCIDENTAL_1: -6,
-                       const.ACCIDENTAL_2: -2}
+accidental_y_offset = {ACCIDENTAL__2: -7,
+                       ACCIDENTAL__1: -7,
+                       ACCIDENTAL_0: -5,
+                       ACCIDENTAL_1: -6,
+                       ACCIDENTAL_2: -2}
 
 class Engraver:
     def __init__(self):
@@ -152,8 +171,6 @@ class AccidentalsEngraver(Engraver):
 class KeySignatureEngraver(Engraver):
     def __init__(self, old_key, key, clef):
         Engraver.__init__(self)
-        self.m_old_key = old_key
-        self.m_key = key
         self.m_clef = clef
         self.m_old_accidentals = mpdutils.key_to_accidentals(old_key)
         self.m_accidentals = mpdutils.key_to_accidentals(key)
@@ -209,11 +226,11 @@ class NoteheadEngraver(Engraver):
         """
         Engraver.__init__(self)
         if duration.m_nh < 2:
-            self.m_head = const.NOTEHEAD_0
+            self.m_head = NOTEHEAD_WHOLE
         elif duration.m_nh > 2:
-            self.m_head = const.NOTEHEAD_2
+            self.m_head = NOTEHEAD_QUARTER
         else:
-            self.m_head = const.NOTEHEAD_1
+            self.m_head = NOTEHEAD_HALF
         self.m_shift = shift
         self.m_numdots = duration.m_dots
         self.m_ypos = ypos
@@ -239,13 +256,19 @@ class NoteheadEngraver(Engraver):
                 int(self.m_xpos+dim.xshift*(self.m_shift+1.5+n/2.0)),
                 -3 + staff_yoffset + dim.linespacing*self.m_ypos/2)
     def __str__(self):
-        return "(NoteheadEngraver: xpos:%s, ypos:%i, head:%i, shift=%i)" % (self.m_xpos,
-                                                                    self.m_ypos,
-                                                                    self.m_head,
-                                                                    self.m_shift)
+        try:
+            xpos = self.m_xpos
+        except AttributeError:
+            xpos = None
+        return "(NoteheadEngraver: xpos:%s, ypos:%i, head:%i, shift=%i)" % (
+                xpos, self.m_ypos, self.m_head, self.m_shift)
     def __repr__(self):
+        try:
+            xpos = self.m_xpos
+        except AttributeError:
+            xpos = None
         return "<NoteheadEngraver xpos:%s, ypos:%i, head:%i, shift=%i>" % (
-                self.m_xpos, self.m_ypos, self.m_head, self.m_shift)
+                xpos, self.m_ypos, self.m_head, self.m_shift)
 
 
 class BarlineEngraver(Engraver):
@@ -280,14 +303,14 @@ class TupletEngraver(Engraver):
                 bottom.append(max(s.m_yposes)+5)
         self.m_t = min(top) - 2
         self.m_b = max(bottom) + 2
-        self.m_xpos1 = self.m_stems[0].m_xpos
-        self.m_xpos2 = self.m_stems[-1].m_xpos + dim.xshift
+        self.m_xpos1 = self.m_stems[0].m_xpos - 2
+        self.m_xpos2 = self.m_stems[-1].m_xpos + 2
+        if self.m_stems[-1].m_stemdir == const.DOWN:
+            self.m_xpos2 += dim.xshift
     def engrave(self, widget, gc, staff_yoffset):
         self.do_layout()
         dim = dimentions[self.m_fontsize]
-        m = self.m_stems[0].m_xpos + (self.m_stems[-1].m_xpos -self.m_stems[0].m_xpos) / 2
-        x1 = m - 6
-        x2 = m + 6
+        m = (self.m_xpos1 + self.m_xpos2) / 2
         if self.m_direction == const.UP or self.m_direction == const.BOTH:
             y = min(staff_yoffset - dim.linespacing * 3,
                    staff_yoffset + self.m_t * dim.linespacing / 2)
@@ -296,10 +319,15 @@ class TupletEngraver(Engraver):
             y = max(staff_yoffset + dim.linespacing * 5,
                     staff_yoffset + self.m_b * dim.linespacing /2)
             d = -1
-        widget.window.draw_line(gc, self.m_xpos1, y, x1, y)
-        widget.window.draw_line(gc, x2, y, self.m_xpos2, y)
-        widget.window.draw_line(gc, self.m_xpos1, y, self.m_xpos1, y+5*d)
-        widget.window.draw_line(gc, self.m_xpos2, y, self.m_xpos2, y+5*d)
+        if len(self.m_stems) > 1:
+            # left horiz
+            widget.window.draw_line(gc, self.m_xpos1, y, m - 6, y)
+            # right horiz
+            widget.window.draw_line(gc, m + 6, y, self.m_xpos2, y)
+            # left vertic
+            widget.window.draw_line(gc, self.m_xpos1, y, self.m_xpos1, y+5*d)
+            # right vertic
+            widget.window.draw_line(gc, self.m_xpos2, y, self.m_xpos2, y+5*d)
         cc = widget.create_pango_context()
         ll = pango.Layout(cc)
         ll.set_text(str(self.m_ratio.m_den))
@@ -307,6 +335,7 @@ class TupletEngraver(Engraver):
         sx, sy = [i / pango.SCALE for i in ll.get_size()]
         widget.window.draw_layout(gc, m - sx / 2, y - sy / 2, ll)
 
+
 class BeamEngraver(Engraver):
     def __init__(self):
         Engraver.__init__(self)
@@ -373,7 +402,7 @@ class BeamEngraver(Engraver):
         x2 = self.m_stems[-1].m_xpos
         if self.m_stems[0].m_beamed_stem_top % 2 == 1:
             for x in range(len(self.m_stems)):
-                self.m_stems[x].m_beamed_stem_top =\
+                self.m_stems[x].m_beamed_stem_top = \
                     self.m_stems[x].m_beamed_stem_top - d
         y1 = self.m_stems[0].m_beamed_stem_top * dim.linespacing/2 + staff_yoffset
         beamw = 3
@@ -454,7 +483,11 @@ class StemEngraver(Engraver):
         TODO: if we readd m_stempos, it has to be taken into consideration here.
         """
         if self.m_stemdir == const.UP:
-            self.m_xpos = self.m_xpos + dimentions[self.m_fontsize].xshift
+            self.m_xpos += dimentions[self.m_fontsize].xshift
+        if [n for n in self.m_noteheads if n.m_shift == -1]:
+            self.m_xpos += dimentions[self.m_fontsize].xshift
+            for n in self.m_noteheads:
+                n.m_xpos += dimentions[self.m_fontsize].xshift
     def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         if self.m_duration.m_nh < 2:
@@ -497,44 +530,49 @@ class StemEngraver(Engraver):
                      yroot * dim.linespacing/2 + staff_yoffset)
     def get_flag(self, stemdir):
         assert self.m_duration.m_nh > 4
-        return {8:{const.UP: const.FLAG_8UP,
-                             const.DOWN: const.FLAG_8DOWN},
-                16: {const.UP: const.FLAG_16UP,
-                               const.DOWN: const.FLAG_16DOWN},
-                32: {const.UP: const.FLAG_32UP,
-                               const.DOWN: const.FLAG_32DOWN},
-                64: {const.UP: const.FLAG_64UP,
-                               const.DOWN: const.FLAG_64DOWN}} \
+        return {8:{const.UP: FLAG_8UP,
+                   const.DOWN: FLAG_8DOWN},
+                16: {const.UP: FLAG_16UP,
+                     const.DOWN: FLAG_16DOWN},
+                32: {const.UP: FLAG_32UP,
+                     const.DOWN: FLAG_32DOWN},
+                64: {const.UP: FLAG_64UP,
+                     const.DOWN: FLAG_64DOWN}} \
                               [self.m_duration.m_nh][stemdir]
     def shift_noteheads(self):
-            # We need to look at all stems at the same timepos in the staff when
-            # shifting note heads. An integer value say if and in which direction
-            # a note head is shifted. The value 0 means that the note head is in
-            # its default position. In default position, the noteheads
-            # on stems pointing UP are to the left of the stem and noteheads on
-            # stems pointing DOWN on the right of the stem. And they are all
-            # aligned below each other.
-            #
-            # A positive shift value means that the head is moved to the right,
-            # and a negative is to the left. The unit is notehead-widths, so
-            # the common (and for now only used) value is -1 for noteheads on
-            # down-stems and +1 for noteheads on up-stems.
-            #
+        """
+        We need to look at all stems at the same timepos in the staff when
+        shifting note heads. An integer value say if and in which direction
+        a note head is shifted. The value 0 means that the note head is in
+        its default position. In default position, the noteheads
+        on stems pointing UP are to the left of the stem and noteheads on
+        stems pointing DOWN on the right of the stem. And they are all
+        aligned below each other.
+
+        A positive shift value means that the head is moved to the right,
+        and a negative is to the left. The unit is notehead-widths, so
+        the common (and for now only used) value is -1 for noteheads on
+        down-stems and +1 for noteheads on up-stems.
+        """
         self.m_noteheads.sort(lambda a, b: cmp(a.m_ypos, b.m_ypos))
         # TODO: handle crashing noteheads in two different voices.
+
+        # When laying out noteheads, we start with the notehead at the
+        # end of the stem. That will always be unshifted.
         if self.m_stemdir == const.UP:
             heads = list(reversed(self.m_noteheads))
         else:
             heads = self.m_noteheads
         last_ypos = None
         for idx, head in enumerate(heads):
-            if last_ypos is not None and head.m_ypos == last_ypos + 1 and heads[idx - 1].m_shift == 0:
-                head.m_shift = 1 if self.m_stemdir == const.UP else -1
+            if last_ypos is not None:
+                if abs(last_ypos - head.m_ypos) == 1 and heads[idx - 1].m_shift == 0:
+                    head.m_shift = 1 if self.m_stemdir == const.UP else -1
             last_ypos = head.m_ypos
     def __str__(self):
         return "(StemEngraver)"
     def __repr__(self):
-        return "<NoteheadEngraver xpos:%s>" % (
+        return "<StemEngraver xpos:%s>" % (
                 self.m_xpos, )
 
 
@@ -616,8 +654,20 @@ class _StaffCommon(list):
         """
         list.__init__(self)
         self.m_label = getattr(staff, 'm_label', None)
+        self.m_engravers = {}
+        # When joining two empty Scores with Score.concat2, we need to return
+        # to avoid IndexErrors further down this method.
+        if not staff.w_parent().m_bars:
+            return
         # make a set of all timeposes in the staffs voices
         t = set()
+        # We need to add the timepos of the beginning of all bars, since
+        # all staffs has to display the time signature if it changes. Normally
+        # it is not necessary to do this here, since staff.m_tdict will have
+        # the timepos. But it is necessary for Scores created by Score.concat2
+        [t.add(b.m_timepos) for b in staff.w_parent().m_bars]
+        # Then we add the timepos of all notes and rests
+        [t.add(tp) for tp in staff.m_tdict]
         for voice in staff.m_voices:
             [t.add(tp) for tp in voice.m_tdict]
         if last_timepos is None:
@@ -626,8 +676,6 @@ class _StaffCommon(list):
         else:
             timeposes = [x for x in sorted(t) if x < last_timepos]
         #
-        self.m_engravers = {}
-        fontsize = 20
         clef = None
         keysig = ("c", "major")
         self.refill_accidentals_info(keysig)
@@ -655,6 +703,9 @@ class _StaffCommon(list):
             if (timepos in staff.m_tdict
                     and 'properties' in staff.m_tdict[timepos]):
                 props.update(staff.m_tdict[timepos]['properties'])
+            # Forget accidentals at bar lines
+            if timepos == staff.w_parent().m_bars[bar_idx].m_timepos:
+                self.refill_accidentals_info(keysig)
             ########
             # Clef #
             ########
@@ -719,7 +770,9 @@ class _StaffCommon(list):
                             voice.m_beam = BeamEngraver()
                             beams.append(voice.m_beam)
                             self.append(voice.m_beam)
-                        if elem.m_tupletinfo == 'start':
+                        # If the tuplet contain only one tone, then elem.m_tupletinfo == 'end' and
+                        # voice.m_tuplet will be None
+                        if elem.m_tupletinfo == 'start' or (elem.m_tupletinfo == 'end' and voice.m_tuplet == None):
                             voice.m_tuplet = TupletEngraver(elem.m_tuplet_ratio, elem.m_tuplet_dir)
                     if 'elem' not in eng:
                         eng['elem'] = []
@@ -795,8 +848,8 @@ class StaffContext(_StaffCommon):
             self.m_accidentals_info[step] = 0
         for a in mpdutils.key_to_accidentals(key):
             n = MusicalPitch.new_from_notename(a)
-            for oct in range(-4, 7):
-                n.m_octave_i = oct
+            for octave in range(-4, 7):
+                n.m_octave_i = octave
                 if n.semitone_pitch() < 128:
                     if a[-4:] == 'eses':
                         self.m_accidentals_info[n.steps()] = -2
@@ -815,22 +868,22 @@ class StaffContext(_StaffCommon):
                 return [0, m.m_accidental_i]
             self.m_accidentals_info[steps] = m.m_accidental_i
             return [m.m_accidental_i]
-    def create_notehead_engraver(self, clef, elems):
+    def create_notehead_engraver(self, clef, elements):
         """
         Return a two item
         1: Return a list of the engravers that draw the note heads
         2: The stem engraver
-        Will return the engravers in the same order as the notes in elems.
+        Will return the engravers in the same order as the notes in elements.
         """
         noteheads = []
-        for elem in elems:
+        for elem in elements:
             noteheads.append(NoteheadEngraver(
             0, # shift,
             clef.steps_to_ylinepos(elem.m_musicalpitch.steps()),  # ypos,
             elem.m_musicalpitch,
             elem.m_duration))
         return (noteheads,
-            StemEngraver(noteheads, elems[0].m_duration, elems.m_stemdir, False))
+            StemEngraver(noteheads, elements[0].m_duration, elements.m_stemdir, False))
 
 
 class RhythmStaffContext(_StaffCommon):
@@ -906,7 +959,6 @@ class ScoreContext(object):
                         context.m_engravers[timepos][s].m_xpos = xpos
                         max_width = max(max_width, context.m_engravers[timepos][s].get_width())
                 return max_width
-                xpos += max_width
             xpos += do_col('barline', xpos)
             xpos += do_col('clef', xpos)
             xpos += do_col('keysig', xpos)
