@@ -275,7 +275,7 @@ class MidiEventStream(object):
                         if e.m_pitch not in track_notes[idx]:
                             # This could happen if the user adds extra NoteOffEvents or adds one
                             # with the wrong pitch.
-                            logging.debug("not stopping, not playing now: %s" % e)
+                            logging.debug("not stopping, not playing now: %s", e)
                             continue
                         chn = track_notes[idx][e.m_pitch]
                         del track_notes[idx][e.m_pitch]
@@ -292,7 +292,7 @@ class MidiEventStream(object):
                         elif isinstance(e, TempoEvent):
                             tempo_request = e
                         else:
-                            logging.debug("MidiEventStream: NOT HANDLING EVENT: %s" % e)
+                            logging.debug("MidiEventStream: NOT HANDLING EVENT: %s", e)
             for idx, track in enumerate(tracks2):
                 if tpos in track:
                     for e in track[tpos]['NoteOnEvents']:
@@ -316,7 +316,7 @@ class MidiEventStream(object):
                                 ch_dev.set_channel_data(chn, 'volume', track_state[idx]['volume-requested'])
                                 yield self.VOLUME, chn, track_state[idx]['volume-requested']
                         if ch_dev.is_playing(chn, e.m_pitch):
-                            logging.debug("MidiEventStream: ignoring duplicate tone: %s" % e)
+                            logging.debug("MidiEventStream: ignoring duplicate tone: %s", e)
                             continue
                         track_notes[idx][e.m_pitch] = chn
                         # ch_dev must know which tones are sounding on which
