@@ -169,7 +169,7 @@ class DB(object):
             mapping[uuid] = lessonfile.mk_uri(data['filename'])
             count += 1
             callback(_("Files read: %i") % count)
-        logging.info("get_uuid_to_filename_mapping: scanned %i files." % count)
+        logging.info("get_uuid_to_filename_mapping: scanned %i files.", count)
         return count, mapping
     def read_old_data(self, callback):
         """
@@ -202,7 +202,7 @@ class DB(object):
         imp, uuid_to_filename = self.get_uuid_to_filename_mapping(callback)
         def read_session_data(datadir, is_tests, counter):
             if not os.path.isdir(datadir):
-                logging.debug("read_session_data: '%s' does not exist. Returning" % datadir)
+                logging.debug("read_session_data: '%s' does not exist. Returning", datadir)
                 return
             for uuid in os.listdir(datadir):
                 if not (os.path.isdir(os.path.join(datadir, uuid))
@@ -256,7 +256,7 @@ class DB(object):
 
                     if not (saved_hashvalue == bugfix_hash_file(real_lessonfilename, "hash-bug-workaround")
                             or saved_hashvalue == bugfix_hash_file(real_lessonfilename, "hash-bug-workaround2")):
-                        logging.debug("Ignoring statistics for '%s' since the saved hash value does not match." % lessonfilename)
+                        logging.debug("Ignoring statistics for '%s' since the saved hash value does not match.", lessonfilename)
                         continue
 
                 for timestamp in os.listdir(os.path.join(datadir, uuid)):
@@ -301,7 +301,7 @@ class DB(object):
                 os.path.join(filesystem.app_data(), 'testresults'),
                 True, imp)
         self.set_variable("database_version", 2)
-        logging.info("imported statistics for %s lessonfiles" % imp)
+        logging.info("imported statistics for %s lessonfiles", imp)
         self.conn.commit()
     def upgrade_to_version_2(self):
         """
@@ -325,7 +325,7 @@ class DB(object):
                     "values (?, ?, ?)",
                     (fileid, timestamp, 1))
             except sqlite3.IntegrityError, e:
-                logging.error(u"%s, %s, %s" % (e, fileid, timestamp))
+                logging.error(u"%s, %s, %s", e, fileid, timestamp)
         # Insert all practise sessions into sessioninfo
         for fileid, timestamp in self.conn.execute("select distinct fileid, timestamp from sessions"):
             # This can fail because 3.14.11 and earlier had a bug where the same
@@ -336,7 +336,7 @@ class DB(object):
                     "values (?, ?, ?)",
                     (fileid, timestamp, 0))
             except sqlite3.IntegrityError, e:
-                logging.error(u"%s, %s, %s" % (e, fileid, timestamp))
+                logging.error(u"%s, %s, %s", e, fileid, timestamp)
         # Copy all tests from the obsolete "tests" table to "sessions"
         for a, b, c, d in self.conn.execute("select fileid, timestamp, answerkey, guessed from tests"):
             # This can fail because 3.14.11 and earlier had a bug where the same
@@ -347,7 +347,7 @@ class DB(object):
                     "values (?, ?, ?, ?)",
                     (a, b, c, d))
             except sqlite3.IntegrityError, e:
-                logging.error(u"%s, %s, %s" % (e, fileid, timestamp))
+                logging.error(u"%s, %s, %s", e, fileid, timestamp)
         for fileid, filename in self.conn.execute(
                 "select sessioninfo.fileid, lessonfiles.filename "
                 "from sessioninfo, lessonfiles "
@@ -374,8 +374,8 @@ class DB(object):
         Save the test result in the "lessonfiles" table for faster
         access later.
         """
-        logging.debug("cache_new_test_result(%s, %s, %s, %s)"
-            % (filename, timestamp, required, num_questions))
+        logging.debug("cache_new_test_result(%s, %s, %s, %s)",
+            filename, timestamp, required, num_questions)
         fileid = self.get_fileid(filename)
         count_correct = self.conn.execute("select sum(count) from sessions where fileid=? and timestamp=? and answerkey=guessed", (fileid, timestamp)).fetchone()[0]
         if not count_correct:
@@ -404,7 +404,7 @@ class DB(object):
         (bool, float)
         If no test have been made, we return (None, None)
         """
-        logging.debug("get_test_status(%s)" % filename)
+        logging.debug("get_test_status(%s)", filename)
         try:
             fileid = self.get_fileid(filename)
             row = self.conn.execute("select test_passed, test_result "
@@ -424,7 +424,7 @@ class DB(object):
 
         Return doing nothing if the file does not exist.
         """
-        logging.debug("validate_stored_statistics(%s)" % filename)
+        logging.debug("validate_stored_statistics(%s)", filename)
         if not os.path.isfile(lessonfile.uri_expand(filename)):
             logging.debug("validate_stored_statistics: file does not exist.")
             return
@@ -591,8 +591,8 @@ class AbstractStatistics(object):
             q = solfege.db.conn.execute("select answerkey, guessed, sum(count) from sessions where fileid=? and timestamp>? group by answerkey, guessed", (fileid, self.m_timestamp - seconds))
         ret = {}
         for answer, guess, count in q.fetchall():
-            ret.setdefault(self.int_if_int(answer), {})
-            ret[self.int_if_int(answer)][self.int_if_int(guess)] = count
+            ret.setdefault(answer, {})
+            ret[answer][guess] = count
         return ret
     def reset_session(self):
         """
@@ -618,7 +618,9 @@ class AbstractStatistics(object):
         Register that for the question 'question' the user answered 'answer'.
         """
         assert self.m_timestamp
-        # tuples must be converted to str to store them in sqlite.
+        # tuples must be converted to str to store them in sqlite. Integers
+        # and probably some other types will automatically be converted to
+        # strings by sqlite before storing then in the database.
         if isinstance(question, tuple):
             question = str(question)
         if isinstance(answer, tuple):
@@ -759,7 +761,9 @@ class IntervalStatistics(AbstractStatistics):
     def get_keys(self, all_keys=False):
         # FIXME we have to check that all keys are integers, and filter
         # out those that are not. I don't know how, but I ended up with
-        # the key None inserted into the database.
+        # the key None inserted into the database. Without that bug this
+        # whole method would be unnecessary and AbstractStatistics.get_keys
+        # would be enough.
         def isinteger(s):
             try:
                 int(s)
@@ -767,7 +771,7 @@ class IntervalStatistics(AbstractStatistics):
             except ValueError:
                 return False
         v = AbstractStatistics.get_keys(self, all_keys)
-        return [int(x) for x in v if isinteger(x)]
+        return [x for x in v if isinteger(x)]
     def key_to_pretty_name(self, key):
         return utils.int_to_intervalname(int(key), 1, 1)
 
